package com.applerao.urlrewriter.servlet.listener;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.springframework.context.ApplicationContext;
import org.springframework.web.context.WebApplicationContext;

import com.applerao.urlrewriter.URLRewriteEngine;
import com.applerao.urlrewriter.permalink.GenericPermalinkDefinition;
import com.applerao.urlrewriter.permalink.PermalinkManager;

public abstract class URLRewriteListener implements ServletContextListener{

	private Map<String, String> rules = new HashMap<String, String>();
	
	private Map<GenericPermalinkDefinition, String> pds = 
		new HashMap<GenericPermalinkDefinition, String>();
	
	private ApplicationContext context;
	
	public URLRewriteListener(){
		
	}
	
	public URLRewriteListener(Map<String, String> rules){
		this.rules = rules;
	}
	
	public void contextDestroyed(ServletContextEvent sce) {
		
	}

	public void contextInitialized(ServletContextEvent sce) {
		initContext(sce);
		addRules();
		addPDs();
	}
	
	private void initContext(ServletContextEvent sce) {
		ServletContext servletContext = sce.getServletContext();
		this.context = (ApplicationContext) servletContext
			.getAttribute(WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE);
	}

	private void addRules(){
		this.rules = initRules();
		URLRewriteEngine engine = URLRewriteEngine.getInstance();
		Set<Entry<String, String>> ruleSet = this.rules.entrySet();
		for(Entry<String, String> rule:ruleSet){
			engine.addRule(rule.getKey(), rule.getValue());
		}
	}
	
	private void addPDs(){
		this.pds = initPDs();
		if(this.pds != null){
			PermalinkManager.getInstance().setAutoRegisterRewriteRule(false);
			URLRewriteEngine engine = URLRewriteEngine.getInstance();
			Set<Entry<GenericPermalinkDefinition, String>> pdSet = this.pds.entrySet();
			for(Entry<GenericPermalinkDefinition, String> pd:pdSet){
				engine.addRule(pd.getKey().definitionReguarExpression(), pd.getValue());
				PermalinkManager.getInstance().register(pd.getKey());
			}
		}
	}

	abstract public Map<String, String> initRules();
	
	abstract public Map<GenericPermalinkDefinition, String> initPDs();
	
	@SuppressWarnings("unchecked")
	protected<T> T getBean(String beanName){
		return (T)this.context.getBean(beanName);
	}

}
